/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/evp.h>

#ifndef WIN32
    #define strnicmp    strncasecmp
#endif



#define VER         "0.1"
#define BUFFSZ      8192
#define HOST        "HOST="
#define UID         "UID="
#define PWD         "PWD="
#define ACT         "ACT="
#define B64DECX     bb = *bf; \
                    for(; \
                    bb && \
                    ((bb == ' ') || \
                    (bb == '\t') || \
                    (bb == '\r') || \
                    (bb == '\n')); \
                    bf++);



void ipswitch_evp_pwd(unsigned char *pwd, int len);
unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd;
    unsigned char   buff[BUFFSZ + 1],
                    *p,
                    *lim;


    setbuf(stdout, NULL);

    fputs("\n"
        "Ipswitch Ws FTP client passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <site.INI>\n"
            "\n"
            "All the INI files are located in the Sites folder of the Ws-FTP.\n"
            "Usually the Sites folder is available at the following path:\n"
            "\n"
            "  c:\\Program Files\\Common Files\\WS_FTP\\Sites\n"
            "\n", argv[0]);
        fputs("\n   Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("- open file \"%s\"\n", argv[1]);
    fd = fopen(argv[1], "rt");
    if(!fd) std_err();

    while(fgets(buff, BUFFSZ, fd)) {
        if(*buff == '[') {
            fputc('\n', stdout);
            fputs(buff, stdout);

        } else if(!strnicmp(buff, HOST, sizeof(HOST) - 1)) {
            p = buff + sizeof(HOST) - 1;
            if(*p == '\"') {
                lim = strchr(++p, '\"');
                if(lim) *lim = 0;
            }

            printf("  Hostname:   %s\n", p);

        } else if(!strnicmp(buff, UID, sizeof(UID) - 1)) {
            p = buff + sizeof(UID) - 1;
            if(*p == '\"') {
                lim = strchr(++p, '\"');
                if(lim) *lim = 0;
            }

            printf("  Username:   %s\n", p);

        } else if(!strnicmp(buff, PWD, sizeof(PWD) - 1)) {
            p = buff + sizeof(PWD) - 1;
            if(*p == '\"') {
                lim = strchr(++p, '\"');
                if(lim) *lim = 0;
            }

            ipswitch_evp_pwd(p, strlen(p));
            printf("  Password:   %s\n", p);

        } else if(!strnicmp(buff, ACT, sizeof(ACT) - 1)) {
            p = buff + sizeof(ACT) - 1;
            if(*p == '\"') {
                lim = strchr(++p, '\"');
                if(lim) *lim = 0;
            }

            ipswitch_evp_pwd(p, strlen(p));
            printf("  Account:    %s\n", p);
        }
    }

    fclose(fd);

    fputs("\n\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void ipswitch_evp_pwd(unsigned char *pwd, int len) {
    unsigned char       key[] =
                        "\xE1\xF0\xC3\xD2\xA5\xB4\x87\x96\x69\x78\x4B\x5A\x2D\x3C\x0F\x1E"
                        "\x34\x12\x78\x56\xab\x90\xef\xcd";
    EVP_CIPHER_CTX      ctx;

    if(*pwd != '_') return;
    len = Base64Decode(pwd + 1, pwd) - pwd;
    EVP_CIPHER_CTX_init(&ctx);
    EVP_DecryptInit(&ctx, EVP_des_ede3_cbc(), key, key + 16);
    EVP_DecryptUpdate(&ctx, pwd, &len, pwd, len);
    EVP_CIPHER_CTX_cleanup(&ctx);
}



unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret) {
    int             c,
                    d,
                    i,
                    *Base;
    unsigned char   bb,
                    *bf,
                    *rf;

    if(!*buf) {
        *ret = 0;
        return(ret);
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return(ret);
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
    return(rf);
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}

